
#!/usr/bin/env python
#
# This is an sample Python 2.7 AppEngine application that shows how to
#   1) log in a user using the Projectplace OAuth API and
#   2) extract their projects.
#

__author__ = "Christopher Dalid" 


import webapp2


# Add path to third party libraries that are not supported native in Google App Engine Python 2.7
# https://developers.google.com/appengine/docs/python/tools/libraries27
import sys
sys.path.insert(0,'lib')


from appengine_oauth_library import oauth # Using this to create the initial credentials
import ppapiaccess 								# Using this to fetch data from Projectplace





class CredentialsDance(webapp2.RequestHandler):
	"""
	Generate OAuth credentials for the user.
	"""

	def get(self, mode=''):


		# Extract credentials from config file. 
		from ppapiaccess.credentials import credentials_from_cfg
		creds = credentials_from_cfg(filepath='apiaccess.cfg')

		# By default the config file contain application ("consumer") key and
		# secret for the "pp-api-sdk" application.
		# This enables you to access the service in a basic mode with a
		# manual copy-paste-submit-form process. As soon as you are ready 
		# for the next step you register as a developer at api.projectplace.com (free of charge) 
		# and get your own unique key & secret, this will also enable you to specify
		# a callback url and then there is no longer any need for the manual
		# copy-paste-submit-form process :)
		# 
		# Values for "oauth_token_secret" and "oauth_token" are not
		# available in the default configuration file
		# (default_confpath). These values refer to user specific
		# credentials and must be obtained through a OAuth authentication
		# process. 


		client = oauth.ProjectplaceClient(consumer_key=creds['consumer_key'], consumer_secret=creds['consumer_secret'], callback_url='')
		    
		if mode == "login_with_copy_paste":
			# User will be routed to the login/authenticate page provided by api.projectplace.com, after completing that process
			# the user will have to manually copy a value from the address bar to the input field in a form. Instructions are provided.
			
			template = """
			<html><body> 
			<form action="/credentials/verify" method="post">
			<input type="hidden" name="oauth_token" value="%s">
			<div style="padding-left:25px;padding-top:25px;font:medium normal verdana;"><span style="font: normal large bold verdana;">Manual copy-paste-submit-form process to authenticate access to Projectplace service via Public API</span>
			<ol>
			<li>Click on <a href="%s" target="_blank" title="Go to Projectplace authentication">this link</a> (opens in new window) to get an authentication page from Projectplace to connect with your user account </li>
			<li>Fill in your details and then continue to grant permission.</li>
			<li>The authentication page will reload. Copy value of <i>oauth_verifier</i> parameter from the address bar.<br />
			Should look something like this: https://api.projectplace.com/authorize?oauth_token=fad9e1f55e321589efbdfca8a7bfa7dc&oauth_verifier=the-value-to-copy</li>
			<li>Enter the value here: <input type="text" name="oauth_verifier" size="50"></li>
			<li><input type="submit" value="Continue"></li>
			</ol>
			</form>
			<br />
			<i>Once you register an application at api.projectplace.com, you get your own consumer/application key & secret, you setup that application 
			with this callback url: "%s://%s/credentials/verify".<br />
			Then there is no need for this manual copy-paste-submit-form process and you can use the <a href="/credentials/login_with_callback">automated process</a> instead.
			</div></body></html>'
			"""

			authorization_url = client.get_authorization_url()	#https://api.projectplace.com/authorize?oauth_token=<value>
			oauth_token = authorization_url.split('=')[1]		# Extract the oauth_token so we can enumate a callback call
			self.response.write(template%(oauth_token, authorization_url, self.request.scheme, self.request.host))
			return 

		elif mode == 'login_with_callback':
			# If developer has registered his application at api.projectplace.com then this can be used and will not require any 
			# copy+paste from end user. After authentication process at api.projectplace.com the ".../verify" (callback) will be 
			# done automatiacally.
			
			authorization_url = client.get_authorization_url()	#https://api.projectplace.com/authorize?oauth_token=<value>
			return self.redirect(authorization_url)


		elif mode == "verify":
			# After user has logged in using the login/authentication dialogue provided by api.projectplace.com, either automatically
			# (in case developer has registered app and supplied a callback url) or via the manual copy-paste-submit-form process.
			

			self.response.write('<html><body>')
			auth_token = self.request.get("oauth_token")        
			auth_verifier = self.request.get("oauth_verifier")

			user_info = client.get_user_info(auth_token, auth_verifier=auth_verifier)
			
			self.response.write('<H3>Congratulations! You now have access to your data from Projectplace.com.</h3>')
			self.response.write('<p><i>Your identity at Projectplace.com is:<br /> %s (%s)</i></p><br />'%(user_info['name'],user_info['username']))
			self.response.write('<h4>Proceed by storing these credentials:</h4><ul>')
			self.response.write('<li><a href="/credentials/save_creds_in_localstore?user_token=%s&user_secret=%s">Save your credentials locally on your current device (localStorage)</a></li>'%(user_info['access_token'], user_info['access_secret']))
			self.response.write('<li>Save your credentials with your Google User account (accessible from any device)</li></ul>')
			
			self.response.write('<br /><br /><br /><h6><i>Credential details:<br />')
			self.response.write('- user_token: "%s"<br />'%user_info['access_token'])
			self.response.write('- user_secret: "%s"<p /></i></h6>'%user_info['access_secret'])
			
			self.response.write('</body></html>')
			return 

		elif mode == "save_creds_with_google_account":
			# Credentials will be saved in datastore using the google user as key
			pass

		elif mode == 'save_creds_in_localstore':
			# Credentials will be saved in web broser storage
			template = """\
			<html>
			<head>
			<!-- http://www.w3schools.com/html/html5_webstorage.asp -->
			<script>
			function get_session_token(){
				if (localStorage.user_token) {return localStorage.user_token} else return ''; 
			}
			function get_session_secret(){
				if (localStorage.user_secret){return localStorage.user_secret} else return '';
			}
			</script>
			</head>

			<body>
			<form action="/projects" method="post">
			<input type="hidden" id="user_token"  name="user_token"  value="">
			<input type="hidden" id="user_secret" name="user_secret" value="">
			<script>
			if(typeof(Storage)!=="undefined"){
				// Yes! localStorage and sessionStorage support!
				localStorage.user_token = '%(oauth_token)s';
				localStorage.user_secret = '%(oauth_token_secret)s';
				if (get_session_token()=='%(oauth_token)s' && get_session_secret() == '%(oauth_token_secret)s'){
					document.write('You credentials has been succesfully stored. <br />');
				 	document.getElementById('user_token').value=get_session_token();
				 	document.getElementById('user_secret').value=get_session_secret();
				 	document.write('<input type="submit" value="View data from Projectplace">')
					//document.write('<p><a href="/projects?user_token='+get_session_token()+'&user_secret='+get_session_secret()+'">View data for Projects</a></p>');
				}
				else {
					document.write('Failed to save the values to storage :(');
				}
			}
			else{
				document.write('Sorry! Your browser is not capable of storing values (supported in Internet Explorer 8+, Firefox, Opera, Chrome, and Safari).');
			}
			</script>
			</form>
			</body>

			</html>
			 """
			creds['oauth_token'] = self.request.get("user_token")     
			creds['oauth_token_secret'] = self.request.get("user_secret")
			
			self.response.write(template%creds)
			return

		else:
			self.response.write('<html><body>Oops somethine went wrong... :( Not familiar with mode "%s"</body></html>'%mode)
			return


	def post(self, mode=""):
		"""
		This will be used when manually copy pasting oauth_verifier (see mode == 'login')
		and data is posted using a manual form.
		"""
		if mode == 'verify':
			self.get(mode=mode)
		else:
			raise RuntimeError('POST method not configured for this ("%s") mode.'%mode)



class MainHandler(webapp2.RequestHandler):

 	def get(self, mode=""):
    
		
		
		if mode == "projects":

			# Extract application credentials from config file. 
			from ppapiaccess.credentials import credentials_from_cfg
			creds = credentials_from_cfg(filepath='apiaccess.cfg')

			# The case of client side storage of user credentials
			creds['oauth_token'] = self.request.get("user_token")     
			creds['oauth_token_secret'] = self.request.get("user_secret")
			
			# Todo: Extract/check for server side storage of user credentials (in GAE Datastore)
			

			from ppapiaccess import connect, get_me, get_projects
			connect(creds)

			# Print some basic info
			print
			print "Info fetched via API regarding current user:"
			data = get_me()
			self.response.write("<p> - Name: %s %s<br />"%(data['first_name'],data['last_name']))
			self.response.write(" - Email: %s<br />"%data['email'])
			self.response.write(" - Number of available projects: %d<br />"%len(get_projects()))

 			return

		else:
			# Default view
			template= """\
			<html>

			<head>
			<script>
				function get_session_token(){
				  	if (localStorage.user_token) {return localStorage.user_token} else return ''; 
				 }

				function get_session_secret(){
				  	if (localStorage.user_secret){return localStorage.user_secret} else return '';
				 }

				 function del_session_token_secret(){
				 	// will delete the stored values
				 	localStorage.removeItem('user_token');
				 	localStorage.removeItem('user_secret');
				 } 
			</script>
			</head>

			<body>
			<h2>Welcome to pp-api-sdk</h2>
			<img src="http://upload.wikimedia.org/wikipedia/commons/thumb/e/e4/Infobox_info_icon.svg/200px-Infobox_info_icon.svg.png" width="15">
			<div style="font: italic small normal verdana;">
			This is a web application that can speak with Projectplace.com. It's provided as a base for you as developer so 
			you can get started quickly without having to implement basic infrastructure such as handling of authentication (OAuth).<br />
			In addition to handling the authentication it also provides a sample on how to fetch basic data such as User and Projects 
			from Projectplace.com.
			</div>


			<form action="/projects" method="post">
			<input type="hidden" id="user_token"  name="user_token"  value="">
			<input type="hidden" id="user_secret" name="user_secret" value="">

			<script>
			if (get_session_token() != '' && get_session_secret() != ''){
			 	// User as authenticated previously and we have the credentials
			 	document.getElementById('user_token').value=get_session_token();
			 	document.getElementById('user_secret').value=get_session_secret();
			 	document.write('<p><input type="submit" value="View data from Projectplace">')
			 	document.write('<br /><br /><br /><button onclick="del_session_token_secret();return false;"><i>Clear out my credentials from this device</i></button>');
			 } else {
				 // The user need to authenticate
				 template1 = '<div style="padding-left:25px;padding-top:30px;"><span style="font: normal large bold verdana;">Step 1</span> <span style="font: italic medium normal verdana;">(Setup this app to access your projects at Projectplace.)</span><br />'
				 document.write(template1);
				 document.write('<a href="/credentials/login_with_copy_paste"><span style="font: underlined medium bold verdana;">Manual copy-paste-submit-form</span></a> or ');
				 document.write('<a href="/credentials/login_with_callback">Automated process</a> (<i>for automation  <a href="api.projectplace.com">register an application at api.projectplace.com</a> its free and easy)</i></div>');
			 
			 	template2 = '<div style="padding-left:25px;padding-top:30px;"><span style="font: normal large bold verdana;">Step 2</span> <span style="font: italic medium normal verdana;"></span><br /><span style="font: medium normal verdana;color:gray;text-decoration:underline;">Look at your projects!</span></div>';
			 
			 	document.write(template2);
			 }
			 </script>
			 </form>

			</body></html>
			"""
			self.response.write(template)
		

	def post(self, mode=""):
		"""
		This will be used when viewing data from Projectplace
		"""
		if mode == 'projects':
			self.get(mode=mode)
		else:
			raise RuntimeError('POST method not configured for this ("%s") mode.'%mode)
	


application = webapp2.WSGIApplication([
	('/credentials/(.*)', CredentialsDance),
	('/(.*)', MainHandler),
	], 
	debug=True)





